<?php
/**
* +----------------------------------------------------------------------+
* | F Board                                                              |
* +----------------------------------------------------------------------+
* | Copyright (c) 2003 Fguy (http://fguy.com)                            |
* +----------------------------------------------------------------------+
* | This software licensed under the GPL license and is provided "as is" |
* | and is without any warranty.                                         |
* |                                                                      |
* | You need to purchase commercial licenses:                            |
* |                                                                      |
* | 1. If you distribute this Software with your non open source         |
* |    software.                                                         |
* | 2. If you want warranty for this software.                           |
* | 3. If you want to support development.                               |
* |                                                                      |
* | Licenses, Support and Training can be easily purchased online.       |
* +----------------------------------------------------------------------+
* | Author: Kang Tae Hoon <fguy@fguy.com>                                |
* +----------------------------------------------------------------------+
*
* Ű , .
*
* @author fguy <fguy@fguy.com>
* @version 0.5
* @access public
* @package FBoard
*/

define("DEFAULT_GROUP_NAME", "Group");
define("DEFAULT_BOARD_NAME", "Board");

define("DEBUG", true);

class Schema
{
	var $_prefix = "";
	var $_sql;
	var $_messages;

	/**
	 * 
	 *
     * @access public
	 */
	function Schema($settings, $sql)
	{
		$this->_messages = $settings->messages;
		$this->_prefix	= $settings->prefix;
		$this->_sql		= $sql;
	}

	/**
	 * ⺻  ̺ Ѵ. (ġ )
	 *
	 * @param object Connection $connection ͺ̽ 
	 *
	 * @param string $xml_file XML  
	 *
	 * @return boolean  
	 *
     * @access public
	 */	
	function createBasicTablesFromXML($connection, $xml_file = "lib/schema.xml.php")
	{
		require_once "lib/XPath.php";
		// Database Schema Loading...
		$xpath = new XPath($xml_file);
		
		if(!is_object($xpath))
			return false;


		// Table loading
		$i=1;

		for(;$xpath->match("/" . $this->_prefix . "/table[$i]");$i++)
		{
			$table = $xpath->getAttributes("/" . $this->_prefix . "/table[$i]","name");

			$sql = "CREATE TABLE " . $this->_prefix . "_" . $table . " (\n";

			// Ignore "board" and "comment" table (they are dynamic tables)
			if($table != "board" && $table != "comment")
			{
				// Field loading
				$j = 1;
				while($xpath->match("/" . $this->_prefix . "/table[$i]/field-list/field[$j]"))
				{
					$current_path = "/" . $this->_prefix . "/table[$i]/field-list/field[$j]";

					$field = $xpath->getAttributes($current_path);
					$sql .= "\t".$xpath->getData($current_path);
					$sql .= " ".$this->_sql->getFieldType($field['type']);
					if($field['size'])
						$sql .= "(".$field['size'].") ";
					if($field['null'] == "false")
						$sql .= " NOT NULL ";
					if($field['default'])
					{
						if(is_int($field['default']))
							$sql .= " DEFAULT " . $field['default'] . " ";
						else
							$sql .= " DEFAULT '" . $field['default'] . "' ";
					}
					if($field['extra'])
						$sql .=  " ".$field['extra']."";
					$sql .= ",\n";

					$j++;
				}

				// Primary-Key loading
				if($xpath->match("/" . $this->_prefix . "/table[$i]/primary-key"))
				{
					$current_path = "/" . $this->_prefix . "/table[$i]/primary-key";
					$sql .= "\tPRIMARY KEY (" . $xpath->getData($current_path) . "),\n";
				}

				// Index loading
				$j = 1;
				while($xpath->match("/" . $this->_prefix . "/table[$i]/index[$j]"))
				{
					$current_path = "/" . $this->_prefix . "/table[$i]/index[$j]";
					$attributes = $xpath->getAttributes($current_path);
					$sql .= "\tINDEX " . $attributes[name] . "(" . $xpath->getData($current_path) . "),\n";

					$j++;
				}

				// Constraint loading
				$j = 1;
				while($xpath->match("/" . $this->_prefix . "/table[$i]/constraint[$j]"))
				{
					$current_path = "/" . $this->_prefix . "/table[$i]/constraint[$j]";

					$constraint = $xpath->getAttributes($current_path);

					$sql .= "\tCONSTRAINT " . $xpath->getData($current_path) . " FOREIGN KEY (" . $constraint['foreign-key'] . ") REFERENCES " . $this->_prefix . "_" . $constraint['reference-table'] . "(" . $constraint['reference-field'] . "),\n";

					$j++;
				}

				$sql = substr($sql, 0, -2);

				$sql .= "\n);";

				$result = $connection->query($sql);

				if(DB::isError($result))
				{
					alert($result->getMessage(). " on " . __FILE__ . " line " . __LINE__);
				}
			}
		}

		// Inserting Data...
		$i = 1;
		for(;$xpath->match("/" . $this->_prefix . "/data[$i]");$i++)
		{
			$attributes = $xpath->getAttributes("/" . $this->_prefix . "/data[$i]");
			$table = $this->_prefix . "_" . $attributes[table];

			$j = 1;

			// Load Values...
			for(;$xpath->match("/" . $this->_prefix . "/data[$i]/values[$j]");$j++)
			{
				$current_path = "/" . $this->_prefix . "/data[$i]/values[$j]";
				$attributes = $xpath->getAttributes($current_path);
				$values[$attributes[field]] = $xpath->getData($current_path);
			}

			// insert to DB
			$result = $connection->autoExecute($table, $values);

			if(DB::isError($result))
			{
				alert($result->getMessage(). " on " . __FILE__ . " line " . __LINE__);
			}
		}
		return true;
	}

	/**
	 * Խ 
	 *
	 * @param object Connection $connection ͺ̽ 
	 *
	 * @param string $gid Խ  ׷
	 *
	 * @param string $admin Խ 
	 *
	 * @param string $name Խ ̸
	 *
	 * @param string $skin Խ Ų
	 *
	 * @return int $board_id Խ ID
	 *
     * @access public
	 */	
	function createBoard($connection, $gid, $admin, $name = "", $skin = "default_board", $xml_file = "lib/schema.xml.php")
	{
		// Խ ̵
		$board_id = $connection->getOne("select max(id) + 1 from " . $this->_prefix . "_board_config");

		if(DB::isError($board_id) || !$board_id)
			$board_id = "1";

		if($name == "")
			$name = DEFAULT_BOARD_NAME . "(" . $board_id . ")";

		// Խ  
		$values = array("id" => $board_id, "gid" => $gid, "name" => $name , "admin" => $admin, "skin" => $skin);

		$result = $connection->autoExecute($this->_prefix . "_board_config", $values);

		if(DB::isError($result))
		{
			if(DEBUG)
			{
				print_r($result);
				exit;
				alert($result->getMessage(). " on " . __FILE__ . " line " . __LINE__);
			}
			return false;
		}

		// Խ ̺ 
		require_once "lib/XPath.php";

		// Database Schema Loading...
		$xpath = new XPath($xml_file);
		
		if(!is_object($xpath))
			return false;

		// Table loading
		$i=1;

		for(;$xpath->match("/" . $this->_prefix . "/table[$i]");$i++)
		{
			$table = $xpath->getAttributes("/" . $this->_prefix . "/table[$i]","name");

			$sql = "CREATE TABLE " . $this->_prefix . "_" . $board_id . "_" .  $table . " (\n";

			if($table == "board" || $table == "comment")
			{
				// Field loading
				$j = 1;
				while($xpath->match("/" . $this->_prefix . "/table[$i]/field-list/field[$j]"))
				{
					$current_path = "/" . $this->_prefix . "/table[$i]/field-list/field[$j]";

					$field = $xpath->getAttributes($current_path);
					$sql .= "\t".$xpath->getData($current_path);
					$sql .= " ".$this->_sql->getFieldType($field['type']);
					if($field['size'])
						$sql .= "(".$field['size'].") ";
					if($field['null'] == "false")
						$sql .= " NOT NULL ";
					if($field['default'])
					{
						if(is_int($field['default']))
							$sql .= " DEFAULT " . $field['default'] . " ";
						else
							$sql .= " DEFAULT '" . $field['default'] . "' ";
					}
					if($field['extra'])
						$sql .=  " ".$field['extra']."";
					$sql .= ",\n";

					$j++;
				}

				// Primary-Key loading
				if($xpath->match("/" . $this->_prefix . "/table[$i]/primary-key"))
				{
					$current_path = "/" . $this->_prefix . "/table[$i]/primary-key";
					$sql .= "\tPRIMARY KEY (" . $xpath->getData($current_path) . "),\n";
				}

				// Index loading
				$j = 1;
				while($xpath->match("/" . $this->_prefix . "/table[$i]/index[$j]"))
				{
					$current_path = "/" . $this->_prefix . "/table[$i]/index[$j]";
					$attributes = $xpath->getAttributes($current_path);
					$sql .= "\tINDEX " . $attributes[name] . "(" . $xpath->getData($current_path) . "),\n";

					$j++;
				}

				// Constraint loading
				$j = 1;
				while($xpath->match("/" . $this->_prefix . "/table[$i]/constraint[$j]"))
				{
					$current_path = "/" . $this->_prefix . "/table[$i]/constraint[$j]";

					$constraint = $xpath->getAttributes($current_path);

					$sql .= "\tCONSTRAINT " . $xpath->getData($current_path) . "_" . $board_id . " FOREIGN KEY (" . $constraint['foreign-key'] . ") REFERENCES " . $this->_prefix . "_";

					if($constraint['reference-table'] == "board")
						$sql .= $board_id . "_";

					$sql .= $constraint['reference-table'] . "(" . $constraint['reference-field'] . "),\n";

					$j++;
				}

				$sql = substr($sql, 0, -2);

				$sql .= "\n);";

				$result = $connection->query($sql);

				if(DB::isError($result))
				{
					if(DEBUG)
						alert($result->getMessage(). " on " . __FILE__ . " line " . __LINE__);
					return false;
				}
			}
		}

		//   
		$values = array("id" => $board_id, "grant_read" => "0", "grant_post" => "0", "grant_reply" => "0", "grant_delete" => "3", "grant_comment" => "0", "grant_full_html" => "3", "grant_notice" => "3",  "grant_download" => "0", "grant_full_upload" => "3");

		$result = $connection->autoExecute($this->_prefix . "_board_privileges", $values);

		if(DB::isError($result))
		{
			if(DEBUG)
				alert($result->getMessage(). " on " . __FILE__ . " line " . __LINE__);
			return false;
		}


		// з  
/*
		$values = array("board_id" => $board_id, "num" => 0, "label" => $this->_messages[etc]);

		$result = $connection->autoExecute($this->_prefix . "_category", $values);
*/

		$result = $connection->query("insert into " . $this->_prefix . "_category(id, num, label) values(" . $board_id . ", 0, '" . $this->_messages[etc] . "')");

		if(DB::isError($result))
		{
			if(DEBUG)
				alert($result->getMessage(). " on " . __FILE__ . " line " . __LINE__);
			return false;
		}

		// PHASE  
		$values = array("id" => $board_id, "phase" => 1, "cnt" => 0);

		$result = $connection->autoExecute($this->_prefix . "_phase", $values);

		if(DB::isError($result))
		{
			if(DEBUG)
				alert($result->getMessage(). " on " . __FILE__ . " line " . __LINE__);
			return false;
		}

		return $board_id;
	}

	/**
	 * Խ 
	 *
	 * @param object Connection $connection ͺ̽ 
	 *
	 * @param int $board_id  Խ ID
	 *
	 * @return boolean  
	 *
     * @access public
	 */	

	function deleteBoard($connection, $board_id)
	{
		// Խ ̺ 
		$result = $connection->query("drop table " . $this->_prefix . "_" . $board_id . "_board");
		
		if(DB::isError($result))
		{
			if(DEBUG)
				alert($result->getMessage(). " on " . __FILE__ . " line " . __LINE__);
			return false;
		}

		// ڸƮ ̺ 
		$result = $connection->query("drop table " . $this->_prefix . "_" . $board_id . "_comment");
		
		if(DB::isError($result))
		{
			if(DEBUG)
				alert($result->getMessage(). " on " . __FILE__ . " line " . __LINE__);
			return false;
		}

		// Խ  ڵ 
		$result = $connection->query("delete from " . $this->_prefix . "_board_config where id = " .$board_id);

		if(DB::isError($result))
		{
			if(DEBUG)
				alert($result->getMessage(). " on " . __FILE__ . " line " . __LINE__);
			return false;
		}

		//  ڵ 
		$result = $connection->query("delete from " . $this->_prefix . "_board_privileges where id = " .$board_id);

		if(DB::isError($result))
		{
			if(DEBUG)
				alert($result->getMessage(). " on " . __FILE__ . " line " . __LINE__);
			return false;
		}

		// з ڵ 
		$result = $connection->query("delete from " . $this->_prefix . "_category where id = " .$board_id);

		if(DB::isError($result))
		{
			if(DEBUG)
				alert($result->getMessage(). " on " . __FILE__ . " line " . __LINE__);
			return false;
		}

		// ÷  ڵ 
		$result = $connection->query("delete from " . $this->_prefix . "_files where board_id = " .$board_id);

		if(DB::isError($result))
		{
			if(DEBUG)
				alert($result->getMessage(). " on " . __FILE__ . " line " . __LINE__);
			return false;
		}

		// PHASE ڵ 
		$result = $connection->query("delete from " . $this->_prefix . "_phase where id = " .$board_id);

		if(DB::isError($result))
		{
			if(DEBUG)
				alert($result->getMessage(). " on " . __FILE__ . " line " . __LINE__);
			return false;
		}

		return true;
	}

	/**
	 * ׷ 
	 *
	 * @param object Connection $connection ͺ̽ 
	 *
	 * @param string $parent ׷   ׷ (  ,  1 (ֻ ׷) )
	 *
	 * @param string $name ׷ ̸ 
	 *
	 * @param int $access_level Խ ٱ (0 :  , 1 :   , 2 : ׷  )
	 *
	 * @param int $allow_parent θ    . 0 :   , 1 :  (  )
	 *
	 * @return int ׷ ID
	 *
     * @access public
	 */	
	function createGroup($connection, $parent = "1", $name = "", $access_level = "0", $allow_parent = "0")
	{
		// ׷ ̵
		$group_id = $connection->getOne("select max(gid) + 1 from " . $this->_prefix . "_group");

		if(DB::isError($group_id) || !$group_id)
			$group_id = 0;


		if($name == "")
			$name = DEFAULT_GROUP_NAME . "(" . $group_id . ")";

		//  ׷  
		$values = array("gid" => $group_id, "parent" => $parent, "name" => $name , "access_level" => $access_level, "allow_parent" => $allow_parent);

		$result = $connection->autoExecute($this->_prefix . "_group", $values);

		if(DB::isError($result))
		{
			// alert($result->getMessage(). " on " . __FILE__ . " line " . __LINE__);
			return false;
		}

		// ׷   
		$values = array("gid" => $group_id, "email" => Session::get("email"), "authorized" => "1");

		$result = $connection->autoExecute($this->_prefix . "_group_members", $values);

		if(DB::isError($result))
		{
			// alert($result->getMessage(). " on " . __FILE__ . " line " . __LINE__);
			return false;
		}

		return $group_id;
	}

	/**
	 * ׷ 
	 *
	 * @param object Connection $connection ͺ̽ 
	 *
	 * @param int $group_id  ׷ ID
	 *
	 * @return boolean  
	 *
     * @access public
	 */	

	function deleteGroup($connection, $group_id)
	{
		$result = $connection->query("delete from " . $this->_prefix . "_group where gid = " .$group_id);

		if(DB::isError($result))
		{
			// alert($result->getMessage(). " on " . __FILE__ . " line " . __LINE__);
			return false;
		}

		$result = $connection->query("delete from " . $this->_prefix . "_group_members where gid = " .$group_id);

		if(DB::isError($result))
		{
			// alert($result->getMessage(). " on " . __FILE__ . " line " . __LINE__);
			return false;
		}

		return true;
	}
}